@@ -1,5 +1,5 @@
 use eyre::eyre;
-use fenris::allocators::{SmallDimAllocator, TriDimAllocator};
+use fenris::allocators::{DimAllocator, TriDimAllocator};
 use fenris::assembly::global::{
     apply_homogeneous_dirichlet_bc_csr, apply_homogeneous_dirichlet_bc_rhs, color_nodes, CsrAssembler, CsrParAssembler,
     VectorAssembler, VectorParAssembler,
@@ -74,7 +74,7 @@ where
     D: SmallDim,
     C: ElementConnectivity<f64, GeometryDim = D, ReferenceDim = D> + Sync,
     Source: SourceFunction<f64, D, SolutionDim = U1, Parameters = ()> + Sync,
-    DefaultAllocator: SmallDimAllocator<f64, D>,
+    DefaultAllocator: DimAllocator<f64, D>,
     <DefaultAllocator as Allocator<f64, D>>::Buffer: Sync,
 {
     let (weights, points) = quadrature;
@@ -171,9 +171,9 @@ where
     C: ElementConnectivity<f64, GeometryDim = D, ReferenceDim = D> + Sync,
     D: SmallDim,
     Source: SourceFunction<f64, D, SolutionDim = U1, Parameters = ()> + Sync,
-    // TODO: We should technically only require SmallDimAllocator<_, D>, but Rust gets type
+    // TODO: We should technically only require DimAllocator<_, D>, but Rust gets type
     // inference wrong without this bound...
-    DefaultAllocator: TriDimAllocator<f64, D, D, U1>,
+    DefaultAllocator: TriDimAllocator<f64, U1, D, D>,
     <DefaultAllocator as Allocator<f64, D>>::Buffer: Sync,
 {
     let (a, b) = assemble_linear_system(&mesh, quadrature, poisson_source_function).unwrap();
@@ -212,7 +212,7 @@ pub fn solve_and_produce_output<C, D, Source>(
     C: VtkCellConnectivity + ElementConnectivity<f64, GeometryDim = D, ReferenceDim = D> + Sync,
     D: SmallDim,
     Source: SourceFunction<f64, D, SolutionDim = U1, Parameters = ()> + Sync,
-    // TODO: We should technically only require SmallDimAllocator<_, D>, but Rust gets type
+    // TODO: We should technically only require DimAllocator<_, D>, but Rust gets type
     // inference wrong without this bound...
     DefaultAllocator: TriDimAllocator<f64, D, D, U1>,
     <DefaultAllocator as Allocator<f64, D>>::Buffer: Sync,
